@@ -5,31 +5,44 @@
 
     LaTeX builder.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
+import warnings
 from os import path
 
-from six import iteritems
+from six import text_type
+
 from docutils import nodes
 from docutils.io import FileOutput
 from docutils.utils import new_document
 from docutils.frontend import OptionParser
 
 from sphinx import package_dir, addnodes, highlighting
-from sphinx.util import texescape
+from sphinx.deprecation import RemovedInSphinx17Warning
 from sphinx.config import string_classes, ENUM
-from sphinx.errors import SphinxError
+from sphinx.errors import SphinxError, ConfigError
 from sphinx.locale import _
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
+from sphinx.environment.adapters.asset import ImageAdapter
+from sphinx.util import texescape, logging, status_iterator
 from sphinx.util.nodes import inline_all_toctrees
 from sphinx.util.fileutil import copy_asset_file
 from sphinx.util.osutil import SEP, make_filename
-from sphinx.util.console import bold, darkgreen
-from sphinx.writers.latex import LaTeXWriter
+from sphinx.util.console import bold, darkgreen  # type: ignore
+from sphinx.writers.latex import LaTeXWriter, LaTeXTranslator
+
+if False:
+    # For type annotation
+    from typing import Any, Dict, Iterable, List, Tuple, Union  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.config import Config  # NOQA
+
+
+logger = logging.getLogger(__name__)
 
 
 class LaTeXBuilder(Builder):
@@ -39,46 +52,54 @@ class LaTeXBuilder(Builder):
     name = 'latex'
     format = 'latex'
     supported_image_types = ['application/pdf', 'image/png', 'image/jpeg']
+    supported_remote_images = False
+    default_translator_class = LaTeXTranslator
 
     def init(self):
-        self.docnames = []
-        self.document_data = []
-        self.usepackages = []
+        # type: () -> None
+        self.docnames = []          # type: Iterable[unicode]
+        self.document_data = []     # type: List[Tuple[unicode, unicode, unicode, unicode, unicode, bool]]  # NOQA
+        self.usepackages = []       # type: List[unicode]
         texescape.init()
 
     def get_outdated_docs(self):
+        # type: () -> Union[unicode, List[unicode]]
         return 'all documents'  # for now
 
     def get_target_uri(self, docname, typ=None):
+        # type: (unicode, unicode) -> unicode
         if docname not in self.docnames:
             raise NoUri
         else:
             return '%' + docname
 
     def get_relative_uri(self, from_, to, typ=None):
+        # type: (unicode, unicode, unicode) -> unicode
         # ignore source path
         return self.get_target_uri(to, typ)
 
     def init_document_data(self):
+        # type: () -> None
         preliminary_document_data = [list(x) for x in self.config.latex_documents]
         if not preliminary_document_data:
-            self.warn('no "latex_documents" config value found; no documents '
-                      'will be written')
+            logger.warning('no "latex_documents" config value found; no documents '
+                           'will be written')
             return
         # assign subdirs to titles
-        self.titles = []
+        self.titles = []  # type: List[Tuple[unicode, unicode]]
         for entry in preliminary_document_data:
             docname = entry[0]
             if docname not in self.env.all_docs:
-                self.warn('"latex_documents" config value references unknown '
-                          'document %s' % docname)
+                logger.warning('"latex_documents" config value references unknown '
+                               'document %s', docname)
                 continue
-            self.document_data.append(entry)
+            self.document_data.append(entry)  # type: ignore
             if docname.endswith(SEP + 'index'):
                 docname = docname[:-5]
             self.titles.append((docname, entry[2]))
 
     def write_stylesheet(self):
+        # type: () -> None
         highlighter = highlighting.PygmentsBridge(
             'latex', self.config.pygments_style, self.config.trim_doctest_flags)
         stylesheet = path.join(self.outdir, 'sphinxhighlight.sty')
@@ -86,9 +107,10 @@ class LaTeXBuilder(Builder):
             f.write('\\NeedsTeXFormat{LaTeX2e}[1995/12/01]\n')
             f.write('\\ProvidesPackage{sphinxhighlight}'
                     '[2016/05/29 stylesheet for highlighting with pygments]\n\n')
-            f.write(highlighter.get_stylesheet())
+            f.write(highlighter.get_stylesheet())  # type: ignore
 
     def write(self, *ignored):
+        # type: (Any) -> None
         docwriter = LaTeXWriter(self)
         docsettings = OptionParser(
             defaults=self.env.settings,
@@ -106,7 +128,7 @@ class LaTeXBuilder(Builder):
             destination = FileOutput(
                 destination_path=path.join(self.outdir, targetname),
                 encoding='utf-8')
-            self.info("processing " + targetname + "... ", nonl=1)
+            logger.info("processing %s...", targetname, nonl=1)
             toctrees = self.env.get_doctree(docname).traverse(addnodes.toctree)
             if toctrees:
                 if toctrees[0].get('maxdepth') > 0:
@@ -120,7 +142,7 @@ class LaTeXBuilder(Builder):
                 appendices=((docclass != 'howto') and self.config.latex_appendices or []))
             doctree['tocdepth'] = tocdepth
             self.post_process_images(doctree)
-            self.info("writing... ", nonl=1)
+            logger.info("writing... ", nonl=1)
             doctree.settings = docsettings
             doctree.settings.author = author
             doctree.settings.title = title
@@ -128,9 +150,10 @@ class LaTeXBuilder(Builder):
             doctree.settings.docname = docname
             doctree.settings.docclass = docclass
             docwriter.write(doctree, destination)
-            self.info("done")
+            logger.info("done")
 
     def get_contentsname(self, indexfile):
+        # type: (unicode) -> unicode
         tree = self.env.get_doctree(indexfile)
         contentsname = None
         for toctree in tree.traverse(addnodes.toctree):
@@ -141,8 +164,9 @@ class LaTeXBuilder(Builder):
         return contentsname
 
     def assemble_doctree(self, indexfile, toctree_only, appendices):
+        # type: (unicode, bool, List[unicode]) -> nodes.Node
         self.docnames = set([indexfile] + appendices)
-        self.info(darkgreen(indexfile) + " ", nonl=1)
+        logger.info(darkgreen(indexfile) + " ", nonl=1)
         tree = self.env.get_doctree(indexfile)
         tree['docname'] = indexfile
         if toctree_only:
@@ -163,8 +187,8 @@ class LaTeXBuilder(Builder):
             appendix = self.env.get_doctree(docname)
             appendix['docname'] = docname
             largetree.append(appendix)
-        self.info()
-        self.info("resolving references...")
+        logger.info('')
+        logger.info("resolving references...")
         self.env.resolve_references(largetree, indexfile, self)
         # resolve :ref:s to distant tex files -- we can't add a cross-reference,
         # but append the document name
@@ -184,31 +208,31 @@ class LaTeXBuilder(Builder):
         return largetree
 
     def finish(self):
-        # copy image files
-        if self.images:
-            self.info(bold('copying images...'), nonl=1)
-            for src, dest in iteritems(self.images):
-                self.info(' ' + src, nonl=1)
-                copy_asset_file(path.join(self.srcdir, src),
-                                path.join(self.outdir, dest))
-            self.info()
+        # type: () -> None
+        self.copy_image_files()
 
         # copy TeX support files from texinputs
         context = {'latex_engine': self.config.latex_engine}
-        self.info(bold('copying TeX support files...'))
+        logger.info(bold('copying TeX support files...'))
         staticdirname = path.join(package_dir, 'texinputs')
         for filename in os.listdir(staticdirname):
             if not filename.startswith('.'):
                 copy_asset_file(path.join(staticdirname, filename),
                                 self.outdir, context=context)
 
+        # use pre-1.6.x Makefile for make latexpdf on Windows
+        if os.name == 'nt':
+            staticdirname = path.join(package_dir, 'texinputs_win')
+            copy_asset_file(path.join(staticdirname, 'Makefile_t'),
+                            self.outdir, context=context)
+
         # copy additional files
         if self.config.latex_additional_files:
-            self.info(bold('copying additional files...'), nonl=1)
+            logger.info(bold('copying additional files...'), nonl=1)
             for filename in self.config.latex_additional_files:
-                self.info(' ' + filename, nonl=1)
+                logger.info(' ' + filename, nonl=1)
                 copy_asset_file(path.join(self.confdir, filename), self.outdir)
-            self.info()
+            logger.info('')
 
         # the logo is handled differently
         if self.config.latex_logo:
@@ -216,61 +240,66 @@ class LaTeXBuilder(Builder):
                 raise SphinxError('logo file %r does not exist' % self.config.latex_logo)
             else:
                 copy_asset_file(path.join(self.confdir, self.config.latex_logo), self.outdir)
-        self.info('done')
+        logger.info('done')
+
+    def copy_image_files(self):
+        # type: () -> None
+        if self.images:
+            stringify_func = ImageAdapter(self.app.env).get_original_image_uri
+            for src in status_iterator(self.images, 'copying images... ', "brown",
+                                       len(self.images), self.app.verbosity,
+                                       stringify_func=stringify_func):
+                dest = self.images[src]
+                try:
+                    copy_asset_file(path.join(self.srcdir, src),
+                                    path.join(self.outdir, dest))
+                except Exception as err:
+                    logger.warning('cannot copy image file %r: %s',
+                                   path.join(self.srcdir, src), err)
 
 
 def validate_config_values(app):
+    # type: (Sphinx) -> None
     if app.config.latex_toplevel_sectioning not in (None, 'part', 'chapter', 'section'):
-        app.warn('invalid latex_toplevel_sectioning, ignored: %s' %
-                 app.config.latex_toplevel_sectioning)
-        app.config.latex_toplevel_sectioning = None
-
-    if app.config.latex_use_parts:
-        if app.config.latex_toplevel_sectioning:
-            app.warn('latex_use_parts conflicts with latex_toplevel_sectioning, ignored.')
-        else:
-            app.warn('latex_use_parts is deprecated. Use latex_toplevel_sectioning instead.')
-            app.config.latex_toplevel_sectioning = 'part'
-
-    if app.config.latex_use_modindex is not True:  # changed by user
-        app.warn('latex_use_modindex is deprecated. Use latex_domain_indices instead.')
-
-    if app.config.latex_preamble:
-        if app.config.latex_elements.get('preamble'):
-            app.warn("latex_preamble conflicts with latex_elements['preamble'], ignored.")
-        else:
-            app.warn("latex_preamble is deprecated. "
-                     "Define a latex_elements dict with a 'preamble' key.")
-            app.config.latex_elements['preamble'] = app.config.latex_preamble
-
-    if app.config.latex_paper_size != 'letter':
-        if app.config.latex_elements.get('papersize'):
-            app.warn("latex_paper_size conflicts with latex_elements['papersize'], ignored.")
-        else:
-            app.warn("latex_paper_size is deprecated. "
-                     "Define a latex_elements dict with a 'papersize' key.")
-            if app.config.latex_paper_size:
-                app.config.latex_elements['papersize'] = app.config.latex_paper_size + 'paper'
-
-    if app.config.latex_font_size != '10pt':
-        if app.config.latex_elements.get('pointsize'):
-            app.warn("latex_font_size conflicts with latex_elements['pointsize'], ignored.")
-        else:
-            app.warn("latex_font_size is deprecated. "
-                     "Define a latex_elements dict with a 'pointsize' key.")
-            app.config.latex_elements['pointsize'] = app.config.latex_font_size
+        logger.warning('invalid latex_toplevel_sectioning, ignored: %s',
+                       app.config.latex_toplevel_sectioning)
+        app.config.latex_toplevel_sectioning = None  # type: ignore
 
     if 'footer' in app.config.latex_elements:
         if 'postamble' in app.config.latex_elements:
-            app.warn("latex_elements['footer'] conflicts with "
-                     "latex_elements['postamble'], ignored.")
+            logger.warning("latex_elements['footer'] conflicts with "
+                           "latex_elements['postamble'], ignored.")
         else:
-            app.warn("latex_elements['footer'] is deprecated. "
-                     "Use latex_elements['preamble'] instead.")
+            warnings.warn("latex_elements['footer'] is deprecated. "
+                          "Use latex_elements['preamble'] instead.",
+                          RemovedInSphinx17Warning)
             app.config.latex_elements['postamble'] = app.config.latex_elements['footer']
 
+    if app.config.latex_keep_old_macro_names:
+        warnings.warn("latex_keep_old_macro_names is deprecated. "
+                      "LaTeX markup since Sphinx 1.4.5 uses only prefixed macro names.",
+                      RemovedInSphinx17Warning)
+
+    for document in app.config.latex_documents:
+        try:
+            text_type(document[2])
+        except UnicodeDecodeError:
+            raise ConfigError(
+                'Invalid latex_documents.title found (might contain non-ASCII chars. '
+                'Please use u"..." notation instead): %r' % (document,)
+            )
+
+        try:
+            text_type(document[3])
+        except UnicodeDecodeError:
+            raise ConfigError(
+                'Invalid latex_documents.author found (might contain non-ASCII chars. '
+                'Please use u"..." notation instead): %r' % (document,)
+            )
+
 
 def default_latex_engine(config):
+    # type: (Config) -> unicode
     """ Better default latex_engine settings for specific languages. """
     if config.language == 'ja':
         return 'platex'
@@ -279,6 +308,7 @@ def default_latex_engine(config):
 
 
 def default_latex_docclass(config):
+    # type: (Config) -> Dict[unicode, unicode]
     """ Better default latex_docclass settings for specific languages. """
     if config.language == 'ja':
         return {'manual': 'jsbook',
@@ -288,6 +318,7 @@ def default_latex_docclass(config):
 
 
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     app.add_builder(LaTeXBuilder)
     app.connect('builder-inited', validate_config_values)
 
@@ -299,24 +330,16 @@ def setup(app):
                          None)
     app.add_config_value('latex_logo', None, None, string_classes)
     app.add_config_value('latex_appendices', [], None)
-    app.add_config_value('latex_keep_old_macro_names', True, None)
-    # now deprecated - use latex_toplevel_sectioning
-    app.add_config_value('latex_use_parts', False, None)
+    app.add_config_value('latex_keep_old_macro_names', False, None)
+    app.add_config_value('latex_use_latex_multicolumn', False, None)
     app.add_config_value('latex_toplevel_sectioning', None, None, [str])
-    app.add_config_value('latex_use_modindex', True, None)  # deprecated
     app.add_config_value('latex_domain_indices', True, None, [list])
     app.add_config_value('latex_show_urls', 'no', None)
     app.add_config_value('latex_show_pagerefs', False, None)
-    # paper_size and font_size are still separate values
-    # so that you can give them easily on the command line
-    app.add_config_value('latex_paper_size', 'letter', None)
-    app.add_config_value('latex_font_size', '10pt', None)
     app.add_config_value('latex_elements', {}, None)
     app.add_config_value('latex_additional_files', [], None)
 
     app.add_config_value('latex_docclass', default_latex_docclass, None)
-    # now deprecated - use latex_elements
-    app.add_config_value('latex_preamble', '', None)
 
     return {
         'version': 'builtin',
