@@ -49,7 +49,7 @@
     resolved to a Python object, and otherwise it becomes simple emphasis.
     This can be used as the default role to make links 'smart'.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -62,17 +62,28 @@ from six import string_types
 from types import ModuleType
 
 from six import text_type
-from docutils.parsers.rst import directives
+
+from docutils.parsers.rst import Directive, directives
 from docutils.statemachine import ViewList
 from docutils import nodes
 
 import sphinx
 from sphinx import addnodes
-from sphinx.util import import_object, rst
-from sphinx.util.compat import Directive
+from sphinx.environment.adapters.toctree import TocTree
+from sphinx.util import import_object, rst, logging
 from sphinx.pycode import ModuleAnalyzer, PycodeError
 from sphinx.ext.autodoc import Options
 
+if False:
+    # For type annotation
+    from typing import Any, Dict, List, Tuple, Type, Union  # NOQA
+    from docutils.utils import Inliner  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.ext.autodoc import Documenter  # NOQA
+
+logger = logging.getLogger(__name__)
+
 
 # -- autosummary_toc node ------------------------------------------------------
 
@@ -81,6 +92,7 @@ class autosummary_toc(nodes.comment):
 
 
 def process_autosummary_toc(app, doctree):
+    # type: (Sphinx, nodes.Node) -> None
     """Insert items described in autosummary:: to the TOC tree, but do
     not generate the toctree:: list.
     """
@@ -93,7 +105,7 @@ def process_autosummary_toc(app, doctree):
             try:
                 if (isinstance(subnode, autosummary_toc) and
                         isinstance(subnode[0], addnodes.toctree)):
-                    env.note_toctree(env.docname, subnode[0])
+                    TocTree(env).note(env.docname, subnode[0])
                     continue
             except IndexError:
                 continue
@@ -105,11 +117,13 @@ def process_autosummary_toc(app, doctree):
 
 
 def autosummary_toc_visit_html(self, node):
+    # type: (nodes.NodeVisitor, autosummary_toc) -> None
     """Hide autosummary toctree list in HTML output."""
     raise nodes.SkipNode
 
 
 def autosummary_noop(self, node):
+    # type: (nodes.NodeVisitor, nodes.Node) -> None
     pass
 
 
@@ -120,6 +134,7 @@ class autosummary_table(nodes.comment):
 
 
 def autosummary_table_visit_html(self, node):
+    # type: (nodes.NodeVisitor, autosummary_table) -> None
     """Make the first column of the table non-breaking."""
     try:
         tbody = node[0][0][-1]
@@ -138,11 +153,12 @@ def autosummary_table_visit_html(self, node):
 # -- autodoc integration -------------------------------------------------------
 
 class FakeDirective(object):
-    env = {}
+    env = {}  # type: Dict
     genopt = Options()
 
 
 def get_documenter(obj, parent):
+    # type: (Any, Any) -> Type[Documenter]
     """Get an autodoc.Documenter class suitable for documenting the given
     object.
 
@@ -198,13 +214,15 @@ class Autosummary(Directive):
     }
 
     def warn(self, msg):
+        # type: (unicode) -> None
         self.warnings.append(self.state.document.reporter.warning(
             msg, line=self.lineno))
 
     def run(self):
+        # type: () -> List[nodes.Node]
         self.env = env = self.state.document.settings.env
         self.genopt = Options()
-        self.warnings = []
+        self.warnings = []  # type: List[nodes.Node]
         self.result = ViewList()
 
         names = [x.strip().split()[0] for x in self.content
@@ -237,6 +255,7 @@ class Autosummary(Directive):
         return self.warnings + nodes
 
     def get_items(self, names):
+        # type: (List[unicode]) -> List[Tuple[unicode, unicode, unicode, unicode]]
         """Try to import the given names, and return a list of
         ``[(name, signature, summary_string, real_name), ...]``.
         """
@@ -244,7 +263,7 @@ class Autosummary(Directive):
 
         prefixes = get_import_prefixes_from_env(env)
 
-        items = []
+        items = []  # type: List[Tuple[unicode, unicode, unicode, unicode]]
 
         max_item_chars = 50
 
@@ -289,8 +308,7 @@ class Autosummary(Directive):
                 # be cached anyway)
                 documenter.analyzer.find_attr_docs()
             except PycodeError as err:
-                documenter.env.app.debug(
-                    '[autodoc] module analyzer failed: %s', err)
+                logger.debug('[autodoc] module analyzer failed: %s', err)
                 # no source file -- e.g. for builtin and C modules
                 documenter.analyzer = None
 
@@ -333,12 +351,13 @@ class Autosummary(Directive):
         return items
 
     def get_table(self, items):
+        # type: (List[Tuple[unicode, unicode, unicode, unicode]]) -> List[Union[addnodes.tabular_col_spec, autosummary_table]]  # NOQA
         """Generate a proper list of table nodes for autosummary:: directive.
 
         *items* is a list produced by :meth:`get_items`.
         """
         table_spec = addnodes.tabular_col_spec()
-        table_spec['spec'] = 'p{0.5\linewidth}p{0.5\linewidth}'
+        table_spec['spec'] = r'p{0.5\linewidth}p{0.5\linewidth}'
 
         table = autosummary_table('')
         real_table = nodes.table('', classes=['longtable'])
@@ -351,6 +370,7 @@ class Autosummary(Directive):
         group.append(body)
 
         def append_row(*column_texts):
+            # type: (unicode) -> None
             row = nodes.row('')
             for text in column_texts:
                 node = nodes.paragraph('')
@@ -368,7 +388,7 @@ class Autosummary(Directive):
         for name, sig, summary, real_name in items:
             qualifier = 'obj'
             if 'nosignatures' not in self.options:
-                col1 = ':%s:`%s <%s>`\ %s' % (qualifier, name, real_name, rst.escape(sig))
+                col1 = ':%s:`%s <%s>`\\ %s' % (qualifier, name, real_name, rst.escape(sig))  # type: unicode  # NOQA
             else:
                 col1 = ':%s:`%s <%s>`' % (qualifier, name, real_name)
             col2 = summary
@@ -377,9 +397,20 @@ class Autosummary(Directive):
         return [table_spec, table]
 
 
+def strip_arg_typehint(s):
+    # type: (unicode) -> unicode
+    """Strip a type hint from argument definition."""
+    return s.split(':')[0].strip()
+
+
 def mangle_signature(sig, max_chars=30):
+    # type: (unicode, int) -> unicode
     """Reformat a function signature to a more compact form."""
-    s = re.sub(r"^\((.*)\)$", r"\1", sig).strip()
+    # Strip return type annotation
+    s = re.sub(r"\)\s*->\s.*$", ")", sig)
+
+    # Remove parenthesis
+    s = re.sub(r"^\((.*)\)$", r"\1", s).strip()
 
     # Strip strings (which can contain things that confuse the code below)
     s = re.sub(r"\\\\", "", s)
@@ -387,12 +418,12 @@ def mangle_signature(sig, max_chars=30):
     s = re.sub(r"'[^']*'", "", s)
 
     # Parse the signature to arguments + options
-    args = []
-    opts = []
+    args = []  # type: List[unicode]
+    opts = []  # type: List[unicode]
 
     opt_re = re.compile(r"^(.*, |)([a-zA-Z0-9_*]+)=")
     while s:
-        m = opt_re.search(s)
+        m = opt_re.search(s)  # type: ignore
         if not m:
             # The rest are arguments
             args = s.split(', ')
@@ -401,6 +432,13 @@ def mangle_signature(sig, max_chars=30):
         opts.insert(0, m.group(2))
         s = m.group(1)[:-2]
 
+    # Strip typehints
+    for i, arg in enumerate(args):
+        args[i] = strip_arg_typehint(arg)
+
+    for i, opt in enumerate(opts):
+        opts[i] = strip_arg_typehint(opt)
+
     # Produce a more compact signature
     sig = limited_join(", ", args, max_chars=max_chars - 2)
     if opts:
@@ -414,6 +452,7 @@ def mangle_signature(sig, max_chars=30):
 
 
 def limited_join(sep, items, max_chars=30, overflow_marker="..."):
+    # type: (unicode, List[unicode], int, unicode) -> unicode
     """Join a number of strings to one, limiting the length to *max_chars*.
 
     If the string overflows this limit, replace the last fitting item by
@@ -440,11 +479,12 @@ def limited_join(sep, items, max_chars=30, overflow_marker="..."):
 # -- Importing items -----------------------------------------------------------
 
 def get_import_prefixes_from_env(env):
+    # type: (BuildEnvironment) -> List
     """
     Obtain current Python import prefixes (for `import_by_name`)
     from ``document.env``
     """
-    prefixes = [None]
+    prefixes = [None]  # type: List
 
     currmodule = env.ref_context.get('py:module')
     if currmodule:
@@ -461,6 +501,7 @@ def get_import_prefixes_from_env(env):
 
 
 def import_by_name(name, prefixes=[None]):
+    # type: (unicode, List) -> Tuple[unicode, Any, Any, unicode]
     """Import a Python object that has the given *name*, under one of the
     *prefixes*.  The first name that succeeds is used.
     """
@@ -479,6 +520,7 @@ def import_by_name(name, prefixes=[None]):
 
 
 def _import_by_name(name):
+    # type: (str) -> Tuple[Any, Any, unicode]
     """Import a Python object given its full name."""
     try:
         name_parts = name.split('.')
@@ -521,14 +563,15 @@ def _import_by_name(name):
 
 # -- :autolink: (smart default role) -------------------------------------------
 
-def autolink_role(typ, rawtext, etext, lineno, inliner,
-                  options={}, content=[]):
+def autolink_role(typ, rawtext, etext, lineno, inliner, options={}, content=[]):
+    # type: (unicode, unicode, unicode, int, Inliner, Dict, List[unicode]) -> Tuple[List[nodes.Node], List[nodes.Node]]  # NOQA
     """Smart linking role.
 
     Expands to ':obj:`text`' if `text` is an object that can be imported;
     otherwise expands to '*text*'.
     """
     env = inliner.document.settings.env
+    r = None  # type: Tuple[List[nodes.Node], List[nodes.Node]]
     r = env.get_domain('py').role('obj')(
         'obj', rawtext, etext, lineno, inliner, options, content)
     pnode = r[0][0]
@@ -537,21 +580,24 @@ def autolink_role(typ, rawtext, etext, lineno, inliner,
     try:
         name, obj, parent, modname = import_by_name(pnode['reftarget'], prefixes)
     except ImportError:
-        content = pnode[0]
-        r[0][0] = nodes.emphasis(rawtext, content[0].astext(),
-                                 classes=content['classes'])
+        content_node = pnode[0]
+        r[0][0] = nodes.emphasis(rawtext, content_node[0].astext(),
+                                 classes=content_node['classes'])
     return r
 
 
 def get_rst_suffix(app):
+    # type: (Sphinx) -> unicode
     def get_supported_format(suffix):
-        parser_class = app.config.source_parsers.get(suffix)
+        # type: (unicode) -> Tuple[unicode]
+        parser_class = app.registry.get_source_parsers().get(suffix)
         if parser_class is None:
             return ('restructuredtext',)
         if isinstance(parser_class, string_types):
-            parser_class = import_object(parser_class, 'source parser')
+            parser_class = import_object(parser_class, 'source parser')  # type: ignore
         return parser_class.supported
 
+    suffix = None  # type: unicode
     for suffix in app.config.source_suffix:
         if 'restructuredtext' in get_supported_format(suffix):
             return suffix
@@ -560,6 +606,7 @@ def get_rst_suffix(app):
 
 
 def process_generate_options(app):
+    # type: (Sphinx) -> None
     genfiles = app.config.autosummary_generate
 
     if genfiles and not hasattr(genfiles, '__len__'):
@@ -578,16 +625,17 @@ def process_generate_options(app):
 
     suffix = get_rst_suffix(app)
     if suffix is None:
-        app.warn('autosummary generats .rst files internally. '
-                 'But your source_suffix does not contain .rst. Skipped.')
+        logger.warning('autosummary generats .rst files internally. '
+                       'But your source_suffix does not contain .rst. Skipped.')
         return
 
     generate_autosummary_docs(genfiles, builder=app.builder,
-                              warn=app.warn, info=app.info, suffix=suffix,
-                              base_path=app.srcdir)
+                              warn=logger.warning, info=logger.info,
+                              suffix=suffix, base_path=app.srcdir)
 
 
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     # I need autodoc
     app.setup_extension('sphinx.ext.autodoc')
     app.add_node(autosummary_toc,
