@@ -32,7 +32,7 @@ r"""
     The graph is inserted as a PNG+image map into HTML and a PDF in
     LaTeX.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -48,14 +48,19 @@ from six import text_type
 from six.moves import builtins
 
 from docutils import nodes
-from docutils.parsers.rst import directives
+from docutils.parsers.rst import Directive, directives
 
 import sphinx
 from sphinx.ext.graphviz import render_dot_html, render_dot_latex, \
     render_dot_texinfo, figure_wrapper
 from sphinx.pycode import ModuleAnalyzer
 from sphinx.util import force_decode
-from sphinx.util.compat import Directive
+
+if False:
+    # For type annotation
+    from typing import Any, Dict, List, Tuple  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
 
 
 module_sig_re = re.compile(r'''^(?:([\w.]*)\.)?  # module names
@@ -64,6 +69,7 @@ module_sig_re = re.compile(r'''^(?:([\w.]*)\.)?  # module names
 
 
 def try_import(objname):
+    # type: (unicode) -> Any
     """Import a object or module using *name* and *currentmodule*.
     *name* should be a relative name from *currentmodule* or
     a fully-qualified name.
@@ -72,9 +78,9 @@ def try_import(objname):
     """
     try:
         __import__(objname)
-        return sys.modules.get(objname)
+        return sys.modules.get(objname)  # type: ignore
     except ImportError:
-        modname, attrname = module_sig_re.match(objname).groups()
+        modname, attrname = module_sig_re.match(objname).groups()  # type: ignore
         if modname is None:
             return None
         try:
@@ -85,6 +91,7 @@ def try_import(objname):
 
 
 def import_classes(name, currmodule):
+    # type: (unicode, unicode) -> Any
     """Import a class using its fully-qualified *name*."""
     target = None
 
@@ -127,6 +134,7 @@ class InheritanceGraph(object):
     """
     def __init__(self, class_names, currmodule, show_builtins=False,
                  private_bases=False, parts=0):
+        # type: (unicode, str, bool, bool, int) -> None
         """*class_names* is a list of child classes to show bases from.
 
         If *show_builtins* is True, then Python builtins will be shown
@@ -141,13 +149,15 @@ class InheritanceGraph(object):
                                        'inheritance diagram')
 
     def _import_classes(self, class_names, currmodule):
+        # type: (unicode, str) -> List[Any]
         """Import a list of classes."""
-        classes = []
+        classes = []  # type: List[Any]
         for name in class_names:
             classes.extend(import_classes(name, currmodule))
         return classes
 
     def _class_info(self, classes, show_builtins, private_bases, parts):
+        # type: (List[Any], bool, bool, int) -> List[Tuple[unicode, unicode, List[unicode], unicode]]  # NOQA
         """Return name and bases for all classes that are ancestors of
         *classes*.
 
@@ -158,6 +168,7 @@ class InheritanceGraph(object):
         py_builtins = vars(builtins).values()
 
         def recurse(cls):
+            # type: (Any) -> None
             if not show_builtins and cls in py_builtins:
                 return
             if not private_bases and cls.__name__.startswith('_'):
@@ -179,7 +190,7 @@ class InheritanceGraph(object):
             except Exception:  # might raise AttributeError for strange classes
                 pass
 
-            baselist = []
+            baselist = []  # type: List[unicode]
             all_classes[cls] = (nodename, fullname, baselist, tooltip)
             for base in cls.__bases__:
                 if not show_builtins and base in py_builtins:
@@ -196,6 +207,7 @@ class InheritanceGraph(object):
         return list(all_classes.values())
 
     def class_name(self, cls, parts=0):
+        # type: (Any, int) -> unicode
         """Given a class object, return a fully-qualified name.
 
         This works for things I've tested in matplotlib so far, but may not be
@@ -212,8 +224,9 @@ class InheritanceGraph(object):
         return '.'.join(name_parts[-parts:])
 
     def get_all_class_names(self):
+        # type: () -> List[unicode]
         """Get all of the class names involved in the graph."""
-        return [fullname for (_, fullname, _, _) in self.class_info]
+        return [fullname for (_, fullname, _, _) in self.class_info]  # type: ignore
 
     # These are the default attrs for graphviz
     default_graph_attrs = {
@@ -234,13 +247,16 @@ class InheritanceGraph(object):
     }
 
     def _format_node_attrs(self, attrs):
+        # type: (Dict) -> unicode
         return ','.join(['%s=%s' % x for x in sorted(attrs.items())])
 
     def _format_graph_attrs(self, attrs):
+        # type: (Dict) -> unicode
         return ''.join(['%s=%s;\n' % x for x in sorted(attrs.items())])
 
     def generate_dot(self, name, urls={}, env=None,
                      graph_attrs={}, node_attrs={}, edge_attrs={}):
+        # type: (unicode, Dict, BuildEnvironment, Dict, Dict, Dict) -> unicode
         """Generate a graphviz dot graph from the classes that were passed in
         to __init__.
 
@@ -262,7 +278,7 @@ class InheritanceGraph(object):
             n_attrs.update(env.config.inheritance_node_attrs)
             e_attrs.update(env.config.inheritance_edge_attrs)
 
-        res = []
+        res = []  # type: List[unicode]
         res.append('digraph %s {\n' % name)
         res.append(self._format_graph_attrs(g_attrs))
 
@@ -308,6 +324,7 @@ class InheritanceDiagram(Directive):
     }
 
     def run(self):
+        # type: () -> List[nodes.Node]
         node = inheritance_diagram()
         node.document = self.state.document
         env = self.state.document.settings.env
@@ -347,11 +364,13 @@ class InheritanceDiagram(Directive):
 
 
 def get_graph_hash(node):
+    # type: (inheritance_diagram) -> unicode
     encoded = (node['content'] + str(node['parts'])).encode('utf-8')
     return md5(encoded).hexdigest()[-10:]
 
 
 def html_visit_inheritance_diagram(self, node):
+    # type: (nodes.NodeVisitor, inheritance_diagram) -> None
     """
     Output the graph for HTML.  This will insert a PNG with clickable
     image map.
@@ -384,6 +403,7 @@ def html_visit_inheritance_diagram(self, node):
 
 
 def latex_visit_inheritance_diagram(self, node):
+    # type: (nodes.NodeVisitor, inheritance_diagram) -> None
     """
     Output the graph for LaTeX.  This will insert a PDF.
     """
@@ -399,6 +419,7 @@ def latex_visit_inheritance_diagram(self, node):
 
 
 def texinfo_visit_inheritance_diagram(self, node):
+    # type: (nodes.NodeVisitor, inheritance_diagram) -> None
     """
     Output the graph for Texinfo.  This will insert a PNG.
     """
@@ -414,10 +435,12 @@ def texinfo_visit_inheritance_diagram(self, node):
 
 
 def skip(self, node):
+    # type: (nodes.NodeVisitor, inheritance_diagram) -> None
     raise nodes.SkipNode
 
 
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     app.setup_extension('sphinx.ext.graphviz')
     app.add_node(
         inheritance_diagram,
