@@ -5,11 +5,12 @@
 
     Set up math support in source files and LaTeX/text output.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from docutils import nodes, utils
+from docutils.nodes import make_id
 from docutils.parsers.rst import Directive, directives
 
 from sphinx.roles import XRefRole
@@ -59,15 +60,15 @@ class MathDomain(Domain):
 
     def clear_doc(self, docname):
         # type: (unicode) -> None
-        for labelid, (doc, eqno) in list(self.data['objects'].items()):
+        for equation_id, (doc, eqno) in list(self.data['objects'].items()):
             if doc == docname:
-                del self.data['objects'][labelid]
+                del self.data['objects'][equation_id]
 
     def merge_domaindata(self, docnames, otherdata):
         # type: (Iterable[unicode], Dict) -> None
         for labelid, (doc, eqno) in otherdata['objects'].items():
             if doc in docnames:
-                self.data['objects'][labelid] = doc
+                self.data['objects'][labelid] = (doc, eqno)
 
     def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
         # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
@@ -81,8 +82,8 @@ class MathDomain(Domain):
                 return newnode
             else:
                 title = nodes.Text("(%d)" % number)
-                return make_refnode(builder, fromdocname, docname,
-                                    "equation-" + target, title)
+                node_id = make_id('equation-%s' % target)
+                return make_refnode(builder, fromdocname, docname, node_id, title)
         else:
             return None
 
@@ -226,7 +227,8 @@ class MathDirective(Directive):
             node['number'] = eqno
 
             # add target node
-            target = nodes.target('', '', ids=['equation-' + node['label']])
+            node_id = make_id('equation-%s' % node['label'])
+            target = nodes.target('', '', ids=[node_id])
             self.state.document.note_explicit_target(target)
             ret.insert(0, target)
         except UserWarning as exc:
