@@ -8,7 +8,7 @@
     all todos of your project and lists them along with a backlink to the
     original location.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -18,10 +18,20 @@ from docutils.parsers.rst import directives
 import sphinx
 from sphinx.locale import _
 from sphinx.environment import NoUri
+from sphinx.util import logging
 from sphinx.util.nodes import set_source_info
+from sphinx.util.texescape import tex_escape_map
 from docutils.parsers.rst import Directive
 from docutils.parsers.rst.directives.admonitions import BaseAdmonition
 
+if False:
+    # For type annotation
+    from typing import Any, Dict, Iterable, List  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+
+logger = logging.getLogger(__name__)
+
 
 class todo_node(nodes.Admonition, nodes.Element):
     pass
@@ -46,6 +56,7 @@ class Todo(BaseAdmonition):
     }
 
     def run(self):
+        # type: () -> List[nodes.Node]
         if not self.options.get('class'):
             self.options['class'] = ['admonition-todo']
 
@@ -58,17 +69,20 @@ class Todo(BaseAdmonition):
 
         env = self.state.document.settings.env
         targetid = 'index-%s' % env.new_serialno('index')
+        # Stash the target to be retrieved later in latex_visit_todo_node.
+        todo['targetref'] = '%s:%s' % (env.docname, targetid)
         targetnode = nodes.target('', '', ids=[targetid])
         return [targetnode, todo]
 
 
 def process_todos(app, doctree):
+    # type: (Sphinx, nodes.Node) -> None
     # collect all todos in the environment
     # this is not done in the directive itself because it some transformations
     # must have already been run, e.g. substitutions
     env = app.builder.env
     if not hasattr(env, 'todo_all_todos'):
-        env.todo_all_todos = []
+        env.todo_all_todos = []  # type: ignore
     for node in doctree.traverse(todo_node):
         app.emit('todo-defined', node)
 
@@ -80,7 +94,7 @@ def process_todos(app, doctree):
             targetnode = None
         newnode = node.deepcopy()
         del newnode['ids']
-        env.todo_all_todos.append({
+        env.todo_all_todos.append({  # type: ignore
             'docname': env.docname,
             'source': node.source or env.doc2path(env.docname),
             'lineno': node.line,
@@ -89,7 +103,8 @@ def process_todos(app, doctree):
         })
 
         if env.config.todo_emit_warnings:
-            env.warn_node("TODO entry found: %s" % node[1].astext(), node)
+            logger.warning("TODO entry found: %s", node[1].astext(),
+                           location=node)
 
 
 class TodoList(Directive):
@@ -101,15 +116,17 @@ class TodoList(Directive):
     required_arguments = 0
     optional_arguments = 0
     final_argument_whitespace = False
-    option_spec = {}
+    option_spec = {}  # type: Dict
 
     def run(self):
+        # type: () -> List[todolist]
         # Simply insert an empty todolist node which will be replaced later
         # when process_todo_nodes is called
         return [todolist('')]
 
 
 def process_todo_nodes(app, doctree, fromdocname):
+    # type: (Sphinx, nodes.Node, unicode) -> None
     if not app.config['todo_include_todos']:
         for node in doctree.traverse(todo_node):
             node.parent.remove(node)
@@ -119,16 +136,19 @@ def process_todo_nodes(app, doctree, fromdocname):
     env = app.builder.env
 
     if not hasattr(env, 'todo_all_todos'):
-        env.todo_all_todos = []
+        env.todo_all_todos = []  # type: ignore
 
     for node in doctree.traverse(todolist):
+        if node.get('ids'):
+            content = [nodes.target()]
+        else:
+            content = []
+
         if not app.config['todo_include_todos']:
-            node.replace_self([])
+            node.replace_self(content)
             continue
 
-        content = []
-
-        for todo_info in env.todo_all_todos:
+        for todo_info in env.todo_all_todos:  # type: ignore
             para = nodes.paragraph(classes=['todo-source'])
             if app.config['todo_link_only']:
                 description = _('<<original entry>>')
@@ -155,8 +175,13 @@ def process_todo_nodes(app, doctree, fromdocname):
             para += newnode
             para += nodes.Text(desc2, desc2)
 
-            # (Recursively) resolve references in the todo content
             todo_entry = todo_info['todo']
+            # Remove targetref from the (copied) node to avoid emitting a
+            # duplicate label of the original entry when we walk this node.
+            if 'targetref' in todo_entry:
+                del todo_entry['targetref']
+
+            # (Recursively) resolve references in the todo content
             env.resolve_references(todo_entry, todo_info['docname'],
                                    app.builder)
 
@@ -168,30 +193,52 @@ def process_todo_nodes(app, doctree, fromdocname):
 
 
 def purge_todos(app, env, docname):
+    # type: (Sphinx, BuildEnvironment, unicode) -> None
     if not hasattr(env, 'todo_all_todos'):
         return
-    env.todo_all_todos = [todo for todo in env.todo_all_todos
+    env.todo_all_todos = [todo for todo in env.todo_all_todos  # type: ignore
                           if todo['docname'] != docname]
 
 
 def merge_info(app, env, docnames, other):
+    # type: (Sphinx, BuildEnvironment, Iterable[unicode], BuildEnvironment) -> None
     if not hasattr(other, 'todo_all_todos'):
         return
     if not hasattr(env, 'todo_all_todos'):
-        env.todo_all_todos = []
-    env.todo_all_todos.extend(other.todo_all_todos)
+        env.todo_all_todos = []  # type: ignore
+    env.todo_all_todos.extend(other.todo_all_todos)  # type: ignore
 
 
 def visit_todo_node(self, node):
+    # type: (nodes.NodeVisitor, todo_node) -> None
     self.visit_admonition(node)
     # self.visit_admonition(node, 'todo')
 
 
 def depart_todo_node(self, node):
+    # type: (nodes.NodeVisitor, todo_node) -> None
     self.depart_admonition(node)
 
 
+def latex_visit_todo_node(self, node):
+    # type: (nodes.NodeVisitor, todo_node) -> None
+    title = node.pop(0).astext().translate(tex_escape_map)
+    self.body.append(u'\n\\begin{sphinxadmonition}{note}{')
+    # If this is the original todo node, emit a label that will be referenced by
+    # a hyperref in the todolist.
+    target = node.get('targetref')
+    if target is not None:
+        self.body.append(u'\\label{%s}' % target)
+    self.body.append('%s:}' % title)
+
+
+def latex_depart_todo_node(self, node):
+    # type: (nodes.NodeVisitor, todo_node) -> None
+    self.body.append('\\end{sphinxadmonition}\n')
+
+
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     app.add_event('todo-defined')
     app.add_config_value('todo_include_todos', False, 'html')
     app.add_config_value('todo_link_only', False, 'html')
@@ -200,7 +247,7 @@ def setup(app):
     app.add_node(todolist)
     app.add_node(todo_node,
                  html=(visit_todo_node, depart_todo_node),
-                 latex=(visit_todo_node, depart_todo_node),
+                 latex=(latex_visit_todo_node, latex_depart_todo_node),
                  text=(visit_todo_node, depart_todo_node),
                  man=(visit_todo_node, depart_todo_node),
                  texinfo=(visit_todo_node, depart_todo_node))
