@@ -5,7 +5,7 @@
 
     Glue code for the jinja2 templating engine.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -17,18 +17,28 @@ from jinja2 import FileSystemLoader, BaseLoader, TemplateNotFound, \
     contextfunction
 from jinja2.utils import open_if_exists
 from jinja2.sandbox import SandboxedEnvironment
+from typing import Any, Callable, Iterator, Tuple  # NOQA
 
 from sphinx.application import TemplateBridge
 from sphinx.util.osutil import mtimes_of_files
 
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, List, Iterator, Tuple  # NOQA
+    from jinja2.environment import Environment  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.theming import Theme  # NOQA
+
 
 def _tobool(val):
+    # type: (unicode) -> bool
     if isinstance(val, string_types):
         return val.lower() in ('true', '1', 'yes', 'on')
     return bool(val)
 
 
 def _toint(val):
+    # type: (unicode) -> int
     try:
         return int(val)
     except ValueError:
@@ -36,6 +46,7 @@ def _toint(val):
 
 
 def _slice_index(values, slices):
+    # type: (List, int) -> Iterator[List]
     seq = list(values)
     length = 0
     for value in values:
@@ -57,6 +68,7 @@ def _slice_index(values, slices):
 
 
 def accesskey(context, key):
+    # type: (Any, unicode) -> unicode
     """Helper to output each access key only once."""
     if '_accesskeys' not in context:
         context.vars['_accesskeys'] = {}
@@ -68,12 +80,15 @@ def accesskey(context, key):
 
 class idgen(object):
     def __init__(self):
+        # type: () -> None
         self.id = 0
 
     def current(self):
+        # type: () -> int
         return self.id
 
     def __next__(self):
+        # type: () -> int
         self.id += 1
         return self.id
     next = __next__  # Python 2/Jinja compatibility
@@ -86,6 +101,7 @@ class SphinxFileSystemLoader(FileSystemLoader):
     """
 
     def get_source(self, environment, template):
+        # type: (Environment, unicode) -> Tuple[unicode, unicode, Callable]
         for searchpath in self.searchpath:
             filename = path.join(searchpath, template)
             f = open_if_exists(filename)
@@ -97,6 +113,7 @@ class SphinxFileSystemLoader(FileSystemLoader):
             mtime = path.getmtime(filename)
 
             def uptodate():
+                # type: () -> bool
                 try:
                     return path.getmtime(filename) == mtime
                 except OSError:
@@ -113,12 +130,13 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
     # TemplateBridge interface
 
     def init(self, builder, theme=None, dirs=None):
+        # type: (Builder, Theme, List[unicode]) -> None
         # create a chain of paths to search
         if theme:
             # the theme's own dir and its bases' dirs
-            pathchain = theme.get_dirchain()
-            # then the theme parent paths
-            loaderchain = pathchain + theme.themepath
+            pathchain = theme.get_theme_dirs()
+            # the loader dirs: pathchain + the parent directories for all themes
+            loaderchain = pathchain + [path.join(p, '..') for p in pathchain]
         elif dirs:
             pathchain = list(dirs)
             loaderchain = list(dirs)
@@ -151,21 +169,24 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
         self.environment.globals['accesskey'] = contextfunction(accesskey)
         self.environment.globals['idgen'] = idgen
         if use_i18n:
-            self.environment.install_gettext_translations(
-                builder.app.translator)
+            self.environment.install_gettext_translations(builder.app.translator)  # type: ignore  # NOQA
 
-    def render(self, template, context):
+    def render(self, template, context):  # type: ignore
+        # type: (unicode, Dict) -> unicode
         return self.environment.get_template(template).render(context)
 
     def render_string(self, source, context):
+        # type: (unicode, Dict) -> unicode
         return self.environment.from_string(source).render(context)
 
     def newest_template_mtime(self):
+        # type: () -> float
         return max(mtimes_of_files(self.pathchain, '.html'))
 
     # Loader interface
 
     def get_source(self, environment, template):
+        # type: (Environment, unicode) -> Tuple[unicode, unicode, Callable]
         loaders = self.loaders
         # exclamation mark starts search from theme
         if template.startswith('!'):
