@@ -5,7 +5,7 @@
 
     Quickly setup documentation source to work with Sphinx.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import print_function
@@ -35,12 +35,17 @@ from six.moves.urllib.parse import quote as urlquote
 from docutils.utils import column_width
 
 from sphinx import __display_version__, package_dir
-from sphinx.util.osutil import make_filename
-from sphinx.util.console import purple, bold, red, turquoise, \
-    nocolor, color_terminal
+from sphinx.util.osutil import ensuredir, make_filename
+from sphinx.util.console import (  # type: ignore
+    purple, bold, red, turquoise, nocolor, color_terminal
+)
 from sphinx.util.template import SphinxRenderer
 from sphinx.util import texescape
 
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, List, Pattern  # NOQA
+
 TERM_ENCODING = getattr(sys.stdin, 'encoding', None)
 
 DEFAULT_VALUE = {
@@ -64,14 +69,9 @@ EXTENSIONS = ('autodoc', 'doctest', 'intersphinx', 'todo', 'coverage',
 PROMPT_PREFIX = '> '
 
 
-def mkdir_p(dir):
-    if path.isdir(dir):
-        return
-    os.makedirs(dir)
-
-
 # function to get input from terminal -- overridden by the test suite
 def term_input(prompt):
+    # type: (unicode) -> unicode
     print(prompt, end='')
     return input('')
 
@@ -81,6 +81,7 @@ class ValidationError(Exception):
 
 
 def is_path(x):
+    # type: (unicode) -> unicode
     x = path.expanduser(x)
     if path.exists(x) and not path.isdir(x):
         raise ValidationError("Please enter a valid path name.")
@@ -88,17 +89,21 @@ def is_path(x):
 
 
 def allow_empty(x):
+    # type: (unicode) -> unicode
     return x
 
 
 def nonempty(x):
+    # type: (unicode) -> unicode
     if not x:
         raise ValidationError("Please enter some text.")
     return x
 
 
 def choice(*l):
+    # type: (unicode) -> Callable[[unicode], unicode]
     def val(x):
+        # type: (unicode) -> unicode
         if x not in l:
             raise ValidationError('Please enter one of %s.' % ', '.join(l))
         return x
@@ -106,12 +111,14 @@ def choice(*l):
 
 
 def boolean(x):
+    # type: (unicode) -> bool
     if x.upper() not in ('Y', 'YES', 'N', 'NO'):
         raise ValidationError("Please enter either 'y' or 'n'.")
     return x.upper() in ('Y', 'YES')
 
 
 def suffix(x):
+    # type: (unicode) -> unicode
     if not (x[0:1] == '.' and len(x) > 1):
         raise ValidationError("Please enter a file suffix, "
                               "e.g. '.rst' or '.txt'.")
@@ -119,10 +126,12 @@ def suffix(x):
 
 
 def ok(x):
+    # type: (unicode) -> unicode
     return x
 
 
 def term_decode(text):
+    # type: (unicode) -> unicode
     if isinstance(text, text_type):
         return text
 
@@ -144,9 +153,10 @@ def term_decode(text):
 
 
 def do_prompt(d, key, text, default=None, validator=nonempty):
+    # type: (Dict, unicode, unicode, unicode, Callable[[unicode], Any]) -> None
     while True:
         if default is not None:
-            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)
+            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)  # type: unicode
         else:
             prompt = PROMPT_PREFIX + text + ': '
         if PY2:
@@ -178,6 +188,7 @@ def do_prompt(d, key, text, default=None, validator=nonempty):
 
 
 def convert_python_source(source, rex=re.compile(r"[uU]('.*?')")):
+    # type: (unicode, Pattern) -> unicode
     # remove Unicode literal prefixes
     if PY3:
         return rex.sub('\\1', source)
@@ -187,10 +198,12 @@ def convert_python_source(source, rex=re.compile(r"[uU]('.*?')")):
 
 class QuickstartRenderer(SphinxRenderer):
     def __init__(self, templatedir):
+        # type: (unicode) -> None
         self.templatedir = templatedir or ''
         super(QuickstartRenderer, self).__init__()
 
     def render(self, template_name, context):
+        # type: (unicode, Dict) -> unicode
         user_template = path.join(self.templatedir, path.basename(template_name))
         if self.templatedir and path.exists(user_template):
             return self.render_from_file(user_template, context)
@@ -199,6 +212,7 @@ class QuickstartRenderer(SphinxRenderer):
 
 
 def ask_user(d):
+    # type: (Dict) -> None
     """Ask the user for quickstart values missing from *d*.
 
     Values are:
@@ -374,6 +388,7 @@ directly.''')
 
 
 def generate(d, overwrite=True, silent=False, templatedir=None):
+    # type: (Dict, bool, bool, unicode) -> None
     """Generate project based on values in *d*."""
     template = QuickstartRenderer(templatedir=templatedir)
 
@@ -411,11 +426,11 @@ def generate(d, overwrite=True, silent=False, templatedir=None):
         d[key + '_str'] = d[key].replace('\\', '\\\\').replace("'", "\\'")
 
     if not path.isdir(d['path']):
-        mkdir_p(d['path'])
+        ensuredir(d['path'])
 
     srcdir = d['sep'] and path.join(d['path'], 'source') or d['path']
 
-    mkdir_p(srcdir)
+    ensuredir(srcdir)
     if d['sep']:
         builddir = path.join(d['path'], 'build')
         d['exclude_patterns'] = ''
@@ -426,17 +441,20 @@ def generate(d, overwrite=True, silent=False, templatedir=None):
             'Thumbs.db', '.DS_Store',
         ])
         d['exclude_patterns'] = ', '.join(exclude_patterns)
-    mkdir_p(builddir)
-    mkdir_p(path.join(srcdir, d['dot'] + 'templates'))
-    mkdir_p(path.join(srcdir, d['dot'] + 'static'))
+    ensuredir(builddir)
+    ensuredir(path.join(srcdir, d['dot'] + 'templates'))
+    ensuredir(path.join(srcdir, d['dot'] + 'static'))
 
     def write_file(fpath, content, newline=None):
+        # type: (unicode, unicode, unicode) -> None
         if overwrite or not path.isfile(fpath):
-            print('Creating file %s.' % fpath)
+            if 'quiet' not in d:
+                print('Creating file %s.' % fpath)
             with open(fpath, 'wt', encoding='utf-8', newline=newline) as f:
                 f.write(content)
         else:
-            print('File %s already exists, skipping.' % fpath)
+            if 'quiet' not in d:
+                print('File %s already exists, skipping.' % fpath)
 
     conf_path = os.path.join(templatedir, 'conf.py_t') if templatedir else None
     if not conf_path or not path.isfile(conf_path):
@@ -487,6 +505,7 @@ where "builder" is one of the supported builders, e.g. html, latex or linkcheck.
 
 
 def usage(argv, msg=None):
+    # type: (List[unicode], unicode) -> None
     if msg:
         print(msg, file=sys.stderr)
         print(file=sys.stderr)
@@ -503,13 +522,14 @@ For more information, visit <http://sphinx-doc.org/>.
 
 
 def valid_dir(d):
+    # type: (Dict) -> bool
     dir = d['path']
     if not path.exists(dir):
         return True
     if not path.isdir(dir):
         return False
 
-    if set(['Makefile', 'make.bat']) & set(os.listdir(dir)):
+    if set(['Makefile', 'make.bat']) & set(os.listdir(dir)):  # type: ignore
         return False
 
     if d['sep']:
@@ -525,14 +545,15 @@ def valid_dir(d):
         d['dot'] + 'templates',
         d['master'] + d['suffix'],
     ]
-    if set(reserved_names) & set(os.listdir(dir)):
+    if set(reserved_names) & set(os.listdir(dir)):  # type: ignore
         return False
 
     return True
 
 
 class MyFormatter(optparse.IndentedHelpFormatter):
-    def format_usage(self, usage):
+    def format_usage(self, usage):  # type: ignore
+        # type: (str) -> str
         return usage
 
     def format_help(self, formatter):
@@ -545,6 +566,7 @@ class MyFormatter(optparse.IndentedHelpFormatter):
 
 
 def main(argv=sys.argv):
+    # type: (List[str]) -> int
     if not color_terminal():
         nocolor()
 
@@ -681,6 +703,7 @@ def main(argv=sys.argv):
             print('Invalid template variable: %s' % variable)
 
     generate(d, templatedir=opts.templatedir)
+    return 0
 
 
 if __name__ == '__main__':
