@@ -18,3 +18,10 @@ class Class1(object):
     """
     this is Class1
     """
+
+class Class3(object):
+    """
+    this is Class3
+    """
+    class_attr = 42
+    """this is the class attribute class_attr"""
