@@ -2,6 +2,10 @@
 mod2
 """
 
+def decorator(f):
+    return f
+
+@decorator
 def func2(a, b):
     """
     this is func2
@@ -9,6 +13,7 @@ def func2(a, b):
     return a, b
 
 
+@decorator
 class Class2(object):
     """
     this is Class2
