@@ -6,7 +6,7 @@
 
     This script runs the Sphinx unit test suite.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import print_function
@@ -15,8 +15,7 @@ import os
 import sys
 import warnings
 import traceback
-
-from path import path
+import shutil
 
 testroot = os.path.dirname(__file__) or '.'
 sys.path.insert(0, os.path.abspath(os.path.join(testroot, os.path.pardir)))
@@ -24,6 +23,8 @@ sys.path.insert(0, os.path.abspath(os.path.join(testroot, os.path.pardir)))
 # filter warnings of test dependencies
 warnings.filterwarnings('ignore', category=DeprecationWarning, module='site')  # virtualenv
 warnings.filterwarnings('ignore', category=ImportWarning, module='backports')
+warnings.filterwarnings('ignore', category=ImportWarning, module='pkgutil')
+warnings.filterwarnings('ignore', category=ImportWarning, module='pytest_cov')
 warnings.filterwarnings('ignore', category=PendingDeprecationWarning, module=r'_pytest\..*')
 
 # check dependencies before testing
@@ -44,19 +45,20 @@ os.environ['SPHINX_TEST_TEMPDIR'] = \
     os.path.abspath(os.path.join(testroot, 'build')) \
     if 'SPHINX_TEST_TEMPDIR' not in os.environ \
     else os.path.abspath(os.environ['SPHINX_TEST_TEMPDIR'])
-tempdir = path(os.environ['SPHINX_TEST_TEMPDIR'])
+
+tempdir = os.environ['SPHINX_TEST_TEMPDIR']
 print('Temporary files will be placed in %s.' % tempdir)
-if tempdir.exists():
-    tempdir.rmtree()
-tempdir.makedirs()
+if os.path.exists(tempdir):
+    shutil.rmtree(tempdir)
+os.makedirs(tempdir)
 
 print('Running Sphinx test suite (with Python %s)...' % sys.version.split()[0])
 sys.stdout.flush()
 
-# exclude 'root' and 'roots' dirs for pytest test collector
+# exclude 'roots' dirs for pytest test collector
 ignore_paths = [
     os.path.relpath(os.path.join(os.path.dirname(os.path.abspath(__file__)), sub))
-    for sub in ('root', 'roots')
+    for sub in ('roots',)
 ]
 args = sys.argv[1:]
 for ignore_path in ignore_paths:
