@@ -6,7 +6,7 @@
     Test the sphinx.config.Config class and its handling in the
     Application class.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from six import PY3, iteritems
@@ -16,6 +16,7 @@ import mock
 import sphinx
 from sphinx.config import Config
 from sphinx.errors import ExtensionError, ConfigError, VersionRequirementError
+from sphinx.testing.path import path
 
 
 @pytest.mark.sphinx(confoverrides={
@@ -90,7 +91,8 @@ def test_extension_values(app, status, warning):
     assert 'already present' in str(excinfo.value)
 
 
-def test_errors_warnings(tempdir):
+@mock.patch("sphinx.config.logger")
+def test_errors_warnings(logger, tempdir):
     # test the error for syntax errors in the config file
     (tempdir / 'conf.py').write_text(u'project = \n', encoding='ascii')
     with pytest.raises(ConfigError) as excinfo:
@@ -102,8 +104,9 @@ def test_errors_warnings(tempdir):
         u'# -*- coding: utf-8\n\nproject = u"Jägermeister"\n',
         encoding='utf-8')
     cfg = Config(tempdir, 'conf.py', {}, None)
-    cfg.init_values(lambda warning: 1 / 0)
+    cfg.init_values()
     assert cfg.project == u'Jägermeister'
+    assert logger.called is False
 
     # test the warning for bytestrings with non-ascii content
     # bytestrings with non-ascii content are a syntax error in python3 so we
@@ -113,13 +116,10 @@ def test_errors_warnings(tempdir):
     (tempdir / 'conf.py').write_text(
         u'# -*- coding: latin-1\nproject = "fooä"\n', encoding='latin-1')
     cfg = Config(tempdir, 'conf.py', {}, None)
-    warned = [False]
 
-    def warn(msg):
-        warned[0] = True
-
-    cfg.check_unicode(warn)
-    assert warned[0]
+    assert logger.warning.called is False
+    cfg.check_unicode()
+    assert logger.warning.called is True
 
 
 def test_errors_if_setup_is_not_callable(tempdir, make_app):
@@ -130,8 +130,19 @@ def test_errors_if_setup_is_not_callable(tempdir, make_app):
     assert 'callable' in str(excinfo.value)
 
 
+@pytest.fixture
+def make_app_with_empty_project(make_app, tempdir):
+    (tempdir / 'conf.py').write_text('')
+
+    def _make_app(*args, **kw):
+        kw.setdefault('srcdir', path(tempdir))
+        return make_app(*args, **kw)
+    return _make_app
+
+
 @mock.patch.object(sphinx, '__display_version__', '1.3.4')
-def test_needs_sphinx(make_app):
+def test_needs_sphinx(make_app_with_empty_project):
+    make_app = make_app_with_empty_project
     # micro version
     app = make_app(confoverrides={'needs_sphinx': '1.3.3'})  # OK: less
     app.cleanup()
@@ -157,14 +168,16 @@ def test_needs_sphinx(make_app):
         make_app(confoverrides={'needs_sphinx': '2'})  # NG: greater
 
 
-def test_config_eol(tempdir):
+@mock.patch("sphinx.config.logger")
+def test_config_eol(logger, tempdir):
     # test config file's eol patterns: LF, CRLF
     configfile = tempdir / 'conf.py'
     for eol in (b'\n', b'\r\n'):
         configfile.write_bytes(b'project = "spam"' + eol)
         cfg = Config(tempdir, 'conf.py', {}, None)
-        cfg.init_values(lambda warning: 1 / 0)
+        cfg.init_values()
         assert cfg.project == u'spam'
+        assert logger.called is False
 
 
 @pytest.mark.sphinx(confoverrides={'master_doc': 123,
