@@ -5,14 +5,16 @@
 
     Test docutils.conf support for several writers.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+import sys
 
 import pytest
-from util import path, SkipTest
+from sphinx.testing.path import path
+from sphinx.testing.util import SkipTest
 
 
 def regex_count(expr, result):
@@ -71,13 +73,16 @@ def test_texinfo(app, status, warning):
 
 @pytest.mark.sphinx('html', testroot='docutilsconf',
                     docutilsconf='[general]\nsource_link=true\n')
+@pytest.mark.skip(sys.platform == "win32" and \
+                  not (sys.version_info.major >= 3 and sys.version_info.minor >= 2),
+                  reason="Python < 3.2 on Win32 doesn't handle non-ASCII paths right")
 def test_docutils_source_link_with_nonascii_file(app, status, warning):
     srcdir = path(app.srcdir)
     mb_name = u'\u65e5\u672c\u8a9e'
     try:
         (srcdir / (mb_name + '.txt')).write_text('')
     except UnicodeEncodeError:
-        from path import FILESYSTEMENCODING
+        from sphinx.testing.path import FILESYSTEMENCODING
         raise SkipTest(
             'nonascii filename not supported on this filesystem encoding: '
             '%s', FILESYSTEMENCODING)
