@@ -5,7 +5,7 @@
 
     Tests the std domain
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -25,6 +25,7 @@ def test_process_doc_handle_figure_caption():
         nametypes={'testname': True},
         nameids={'testname': 'testid'},
         ids={'testid': figure_node},
+        citation_refs={},
     )
     document.traverse.return_value = []
 
@@ -47,6 +48,7 @@ def test_process_doc_handle_table_title():
         nametypes={'testname': True},
         nameids={'testname': 'testid'},
         ids={'testid': table_node},
+        citation_refs={},
     )
     document.traverse.return_value = []
 
@@ -57,3 +59,21 @@ def test_process_doc_handle_table_title():
     assert 'testname' in domain.data['labels']
     assert domain.data['labels']['testname'] == (
         'testdoc', 'testid', 'title text')
+
+
+def test_get_full_qualified_name():
+    env = mock.Mock(domaindata={})
+    domain = StandardDomain(env)
+
+    # normal references
+    node = nodes.reference()
+    assert domain.get_full_qualified_name(node) is None
+
+    # simple reference to options
+    node = nodes.reference(reftype='option', reftarget='-l')
+    assert domain.get_full_qualified_name(node) is None
+
+    # options with std:program context
+    kwargs = {'std:program': 'ls'}
+    node = nodes.reference(reftype='option', reftarget='-l', **kwargs)
+    assert domain.get_full_qualified_name(node) == 'ls.-l'
